# Copyright 2024-2025 Yaroslav Petrov <yaroslav.v.petrov@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from functools import cache
from aio_pika.robust_connection import (
    AbstractRobustConnection,
    AbstractRobustChannel,
    connect_robust,
)
from aio_pika.pool import Pool


@cache
def connection_pool(amqp_uri: str) -> Pool[AbstractRobustConnection]:
    async def get_connection():
        return await connect_robust(amqp_uri)

    return Pool(get_connection, max_size=2)


AmqpPool = Pool[AbstractRobustChannel]


@cache
def channel_pool(amqp_uri: str) -> AmqpPool:
    async def get_channel():
        async with connection_pool(amqp_uri).acquire() as connection:
            return await connection.channel()

    return Pool(get_channel, max_size=10)
